# ---------------------------------------------------------------------------
# Copyright 2002 Benjamin Trott
# Copyright 2003,2004 Dwight Shih
#
# Functions for storing and retrieving ping data.
# Derived from the Standalone Trackback package released by Benjamin Trott.
#
# This code is released under the Artistic License.
#
# $Id: core.pm,v 1.12 2004/12/01 20:01:11 dwight Exp $
# ---------------------------------------------------------------------------

package trackback::core;

use strict;
use File::Spec::Functions;
use bradchoate::sanitize qw( sanitize_html );
use CGI qw( :cgi );

require Storable;

sub new {
    my $class = shift;
    my %param = @_;
    my $self  = {};
    bless ($self,$class);

    # Directory where all the trackback data will be stored
    if ( $param{DATA_DIRECTORY} ) {
        $self->{DATA_DIRECTORY} = $param{DATA_DIRECTORY};
    } else {
        $self->{DATA_DIRECTORY} = "./data";
    }

    # Maximum number of items to return in a RSS query
    if ( $param{MAX_RSS_ITEMS} ) {
        $self->{MAX_RSS_ITEMS} = $param{MAX_RSS_ITEMS};
    } else {
        $self->{MAX_RSS_ITEMS} = 15;
    }

    # Allowed tags in a trackback excerpt
    #  comma separated list with minimal spaces
    # No tags allowed in title, blog name or url
    if ( $param{ALLOWED_TAGS} ) {
        $self->{ALLOWED_TAGS} = $param{ALLOWED_TAGS};
    } else {
        $self->{ALLOWED_TAGS} = "a href,b,i,strong,em,p";
    }

    # Parameter used to identify the PostId
    if ( $param{ID_PARAMETER} ) {
        $self->{ID_PARAMETER} = $param{ID_PARAMETER};
    } else {
        $self->{ID_PARAMETER}    = "p";
    }

    # Parameter used to identify the Command
    if ( $param{MODE_PARAMETER} ) {
        $self->{MODE_PARAMETER} = $param{MODE_PARAMETER};
    } else {
        $self->{MODE_PARAMETER} = "c";
    }

    # 
    if ( $param{BLOG_NAME} ) {
        $self->{BLOG_NAME} = $param{BLOG_NAME};
    } else {
        $self->{BLOG_NAME} = "This Weblog";
    }

    # 
    if ( $param{BLOG_URL} ) {
        $self->{BLOG_URL} = $param{BLOG_URL};
    } else {
        $self->{BLOG_URL} = "http://localhost/";
    }

    # 
    if ( $param{LANGUAGE} ) {
        $self->{LANGUAGE} = $param{LANGUAGE};
    } else {
        $self->{LANGUAGE} = "en-us";
    }

    # Function used to identify banned outbound links
    if ( $param{BANNED_LINK_FUNCTION} ) {
        $self->{BANNED_LINK_FUNCTION} = $param{BANNED_LINK_FUNCTION};
    }

    # Maximum number of allowed outbound links
    if ( $param{MAXIMUM_LINK_COUNT} ) {
        $self->{MAXIMUM_LINK_COUNT} = $param{MAXIMUM_LINK_COUNT};
    }
    else {
        $self->{MAXIMUM_LINK_COUNT} = 4;
    }

    return $self;
}

#
#
sub data_directory {
    my $self = shift;
    my $value = $self->{DATA_DIRECTORY};
    if ( @_ ) {
        $self->{DATA_DIRECTORY} = shift;
    }
    return $value;
}

sub max_rss_items {
    my $self = shift;
    my $value = $self->{MAX_RSS_ITEMS};
    if ( @_ ) {
        $self->{MAX_RSS_ITEMS} = shift;
    }
    return $value;
}

sub allowed_tags {
    my $self = shift;
    my $value = $self->{ALLOWED_TAGS};
    if ( @_ ) {
        $self->{ALLOWED_TAGS} = shift;
    }
    return $value;
}

sub id_parameter {
    my $self = shift;
    my $value = $self->{ID_PARAMETER};
    if ( @_ ) {
        $self->{ID_PARAMETER} = shift;
    }
    return $value;
}

sub mode_parameter {
    my $self = shift;
    my $value = $self->{MODE_PARAMETER};
    if ( @_ ) {
        $self->{MODE_PARAMETER} = shift;
    }
    return $value;
}

sub blog_url {
    my $self = shift;
    my $value = $self->{BLOG_URL};
    if ( @_ ) {
        $self->{BLOG_URL} = shift;
    }
    return $value;
}

sub language {
    my $self = shift;
    my $value = $self->{LANGUAGE};
    if ( @_ ) {
        $self->{LANGUAGE} = shift;
    }
    return $value;
}

#
# Maintain an associative array that maps data_id to number of pings
#
sub count_update {
    my ($self, $tb_id, $data) = @_;

    my $count_data = $self->count_load();
    my $count = scalar @$data;
    if ( $count eq 0 ) {
        delete $count_data->{$tb_id};
    }
    else {
        $count_data->{$tb_id} = $count;
    }
    $self->count_store( $count_data );
}

sub count_load {
    my $self = shift;
    my $count_file = catfile($self->{DATA_DIRECTORY}, 'tb-count.stor' );
    scalar eval { Storable::retrieve($count_file) } || {};
}

sub count_store {
    my ($self,$count_data) = @_;
    my $count_file = catfile($self->{DATA_DIRECTORY}, 'tb-count.stor' );
    Storable::store($count_data, $count_file);
}

#
#
sub ping_store {
    my ($self,$tb_id,$ping) = @_;

    $ping->{title}     = sanitize_html($ping->{title},"");
    $ping->{blog_name} = sanitize_html($ping->{blog_name},"");
    $ping->{url}       = fix_url( sanitize_html($ping->{url},"") );
    $ping->{excerpt}   = sanitize_html($ping->{excerpt},$self->{ALLOWED_TAGS});

    my $data = $self->data_load($tb_id);
    unshift @$data, $ping;
    $self->summary_add($tb_id, $ping);
    $self->data_store($tb_id, $data);
}

#
#
sub rss_post {
    my ($self, $tb_id) = @_;
    my $data = $self->data_load( $tb_id );
    my $rss = qq(<rss version="0.91"><channel><title>TB: $tb_id</title>\n);
    my $max = $self->{MAX_RSS_ITEMS} ? $self->{MAX_RSS_ITEMS} - 1 : $#$data;
    for my $i (@{$data}[0..$max]) {
        $rss .= sprintf "<item>%s%s%s%s</item>\n",
                xml('title', $i->{title}),
                xml('link', $i->{url}), xml('description', $i->{excerpt}),
                xml('pubDate', POSIX::strftime("%a, %d %b %Y %H:%M:%S %Z", localtime($i->{timestamp}))) if $i;
    }
    $rss . qq(</channel></rss>);
}

sub rss_summary {
    my ($self, $type) = @_;
    my $data = $self->summary_load();
    my $rss = qq(<rss version="0.91"><channel><title>$self->{BLOG_NAME}: $type Summary</title>\n);
    $rss .= qq(<link>$self->{BLOG_URL}</link>\n);
    $rss .= qq(<description>$type Summary RSS</description>\n);
    $rss .= qq(<language>$self->{LANGUAGE}</language>\n);
    for my $i (@$data) {
        $rss .= sprintf "<item>%s%s%s%s</item>\n",
                xml('title', "New $type to $i->{tb_id}: $i->{title}"),
                xml('link', $i->{url}), xml('description', $i->{excerpt}),
                xml('pubDate', POSIX::strftime("%a, %d %b %Y %H:%M:%S %Z", localtime($i->{timestamp}))) if $i;
    }
    $rss . qq(</channel></rss>);
}

#
# Maintain list of the last MAX_RSS_ITEMS pings received (all posts)
# used to generate rss summary for the site
#
sub summary_add {
    my ($self, $tb_id, $data) = @_;
    my $summary_data = $self->summary_load();
    $data->{tb_id} = $tb_id;
    my $size = unshift @$summary_data, $data;
    if ( $size > $self->{MAX_RSS_ITEMS} ) { pop @$summary_data; }
    $self->summary_store($summary_data);
    delete $data->{tb_id};
}

sub summary_load {
    my $self = shift;
    my $summary_file = catfile($self->{DATA_DIRECTORY}, 'tb-summary.stor' );
    scalar eval { Storable::retrieve($summary_file) } || [];
}

sub summary_store {
    my ($self,$summary_data) = @_;
    my $summary_file = catfile($self->{DATA_DIRECTORY}, 'tb-summary.stor' );
    Storable::store($summary_data, $summary_file);
}

#
# Maintain the list of pings received for a post
#
sub data_load {
    my ($self,$tb_id) = @_;
    my $tb_file = catfile( $self->{DATA_DIRECTORY}, $tb_id . '.stor');
    scalar eval { Storable::retrieve($tb_file) } || [];
}

sub data_store {
    my ($self, $tb_id, $data) = @_;
    my $tb_file = catfile( $self->{DATA_DIRECTORY}, $tb_id . '.stor');
    if ( scalar @$data > 0 ) {
        Storable::store( $data,$tb_file );
    } else {
        unlink( $tb_file );
    }
    $self->count_update( $tb_id,$data );
}

#
# Get the Post ID
#  1st see if it's specified as a parameter
#  2nd see if it's specified by the path
#  3rd see if it's part of the request URI (for Radio Userland)
#
sub get_id {
    my $self = shift;
    
    my $tb_id = param($self->{ID_PARAMETER});
    unless ($tb_id) {
        $tb_id = url_param($self->{ID_PARAMETER});
        unless ($tb_id) {
            if (my $pi = path_info()) {
                ($tb_id = $pi) =~ s!^/!!;
            }
        }
    }

    return '' unless $tb_id;
    $tb_id =~ tr/a-zA-Z0-9/_/cs;

    #untaint the id
    if ( $tb_id =~ /(\w+)/ ) {
        return $1;
    } else {
        return '';
    }
}

#
#
sub get_mode {
    my $self = shift;
    return param($self->{MODE_PARAMETER});
}

#
#
sub fix_url {
    my $url = shift;
    if ( $url =~ /^[Hh][Tt][Tt][Pp]:\/\// ) { return $url; }
    if ( $url =~ /^[Ff][Tt][Pp]:\/\// )  { return $url; }
    if ( $url =~ /^\s*$/ ) { return ""; }
    if ( $url =~ /^[^\/]/ )     { return "http://$url"; }
    if ( $url =~ /^\/[^\/]/ )   { return "http:/$url"; }
    if ( $url =~ /^\/\/[^\/]/ ) { return "http:$url"; }
    return "http://$url";
}

#
#
sub has_banned_link {
    my ($self, $text) = @_;
    my ($link_count) = 0;

    $text = sanitize_html( $text,"a href" ); # clean up the text for the regexp
    while ( $text =~ s@<a href=(['"])(\w+:/*)?(www\.)?([^/"']+)[^>]*>@@i ) {
        if ( $self->{BANNED_LINK_FUNCTION} && $self->{BANNED_LINK_FUNCTION}($4) ) {
            return -1;
        }

        $link_count++;
        if ( $link_count > $self->{MAXIMUM_LINK_COUNT} ) {
            return $link_count;
        }
    }

    return 0;
}

#
#
sub exit_with_xml {
    my ($self,$xml_msg) = @_;
    print "Content-Type: text/xml\n\n";
    print qq(<?xml version="1.0" encoding="iso-8859-1"?>\n<response>\n);
    print qq(<error>0</error>\n);
    if ( $xml_msg ) { print $xml_msg; }
    print "</response>\n";
    exit;
}

sub exit_with_error {
    my ($self,$error) = @_;
    print "Content-Type: text/xml\n\n";
    print qq(<?xml version="1.0" encoding="iso-8859-1"?>\n<response>\n);
    print qq(<error>1</error>\n);
    if ( $error ) { print xml('message',$error ); }
    print "</response>\n";
    exit;
}

#
# Utility
#
my(%Map, $RE);
BEGIN {
    %Map = ('&' => '&amp;', '"' => '&quot;', '<' => '&lt;', '>' => '&gt;');
    $RE = join '|', keys %Map;
}

sub xml {
    (my $s = defined $_[1] ? $_[1] : '') =~ s!($RE)!$Map{$1}!g;
    "<$_[0]>$s</$_[0]>\n";
}

1;
